/******* Woodman Docker for CorelDraw Copyright (c) by DukereD 2023 mailto: laserman@tut.by *******/
/*HTML interface functions*/

// Legacy-совместимая функция для получения элементов по классу (для JavaScript 1.5)
// Использование: getElementsByClassName(container, 'className')
function getElementsByClassName(container, className) {
    var elements = [];
    var allElements = container.getElementsByTagName('*');
    
    for (var i = 0; i < allElements.length; i++) {
        var element = allElements[i];
        if (element.className && element.className.indexOf(className) > -1) {
            // Проверяем, что это точное совпадение класса (не подстрока)
            var classes = element.className.split(' ');
            for (var j = 0; j < classes.length; j++) {
                if (classes[j] === className) {
                    elements.push(element);
                    break;
                }
            }
        }
    }
    
    return elements;
}

/**
 * Экранирование специальных символов HTML
 * @param {string} str - строка для экранирования
 * @returns {string} экранированная строка
 */
function escapeHTML(str) {
    if (!str) return "";
    return str.replace(/&/g, '&amp;')
              .replace(/</g, '&lt;')
              .replace(/>/g, '&gt;')
              .replace(/"/g, '&quot;')
              .replace(/'/g, '&#039;');
}

// do_langs_list перенесена в cat_settings.js

// ✅ do_saveversion_list функция заменена на populateSaveVersion в cat_save.js
function do_saveversion_list(){
    // Backward compatibility wrapper
    if (typeof populateSaveVersion === 'function') {
        populateSaveVersion();
    }
}


// do_add_list перенесена в cat_nest.js


// do_color_list перенесена в cat_punktir.js


// do_style_list перенесена в cat_settings.js

function do_style(){
    var style_read = vbRead('css_style');
    var style = (style_read) ? style_read : "style";
    glStyle = (style=="style") ? "" : style + "/";
    if (style!="style") document.write('<link rel="stylesheet" href="css/'+style+'.css" />');
}

function OnLoad(){
    if(in_main){
        tmrs.push({name:"OnLoad start",time:new Date().getTime()});
        if (defdoc()){
            // Инициализация глобальных настроек - радиокнопки читаем отдельно
            var selectedChecked = vbRead("radio_sett_selected");
            var docChecked = vbRead("radio_sett_doc");
            
            // Определяем какая кнопка должна быть активна
            if (docChecked) {
                setVal("radio_sett_doc", true, false);
            } else {
                // По умолчанию или если selected=true
                setVal("radio_sett_selected", true, false); 
            }
            setVal("ch_sett_center", vbRead("ch_sett_center"), false);
            label_radio_sett_selected.innerHTML += lang['radio_sett_selected'];
            label_radio_sett_doc.innerHTML += lang['radio_sett_doc'];
            label_ch_sett_center.innerHTML += lang['ch_sett_center'];
            label_radio_sett_selected.title = hint['radio_sett_selected'];
            label_radio_sett_doc.title = hint['radio_sett_doc'];
            label_ch_sett_center.title = hint['ch_sett_center'];
            btn_glMemory.title = get_hint('btn_glMemory');

            
            // ch_set_titles* настройки теперь читаются ModuleSystem напрямую из vbRead()
            
            tmrs.push({name:"global init done",time:new Date().getTime()});
            
            // Инициализация модульной системы
            if (typeof ModuleSystem !== 'undefined' && ModuleSystem.init) {
                try {
                    ModuleSystem.init();
                } catch (e) {
                    dbg("❌ Ошибка инициализации ModuleSystem: " + e.message);
                }
            }
            
            if(in_main){
                correct_body_size();
            }      

            if (!host) return false;

            setTimeout("stat_send()", 0);
        }else{
            no_doc();
        }

        // Статистика по чтению настроек (для отладки)
        var readact_str = "";
        var readact_cnt = 0;
        for(var key in readact){
            readact_cnt  += readact[key];
            if (readact[key]>1) readact_str += key + ": " + readact[key] + "\n";
        }
        //dbg("Read count: " + readact_str + "\n" + readact_cnt);
        
        loading.style.display = "none";
    }
}

function stat_send(){

    for (var key in vbtr) vbWrite("vbtrans_"+key, vbtr[key]);

    var tm = vbRead('stat_time');
    var now = new Date();
    if (tm==undefined) tm = [77,77]; else tm = tm.split(":");
    if(Math.abs(now.getHours()-tm[1])>1 || now.getDate() != tm[0]){
        vbStatSend(version);
        vbWrite('stat_time',now.getDate() + ":" + now.getHours());
    }
    tmrs.push({name:"stat end",time:new Date().getTime()});

    var timers = "";
    for(var i=1;i<tmrs.length;i++){
        var tm = tmrs[i];
        var time = tm.time - tmrs[i-1].time;
        var timeStr = (time<1000) ? time + "ms" : (time/1000).toFixed(2) + "s";
        timers += tm.name + ": " + timeStr + "\n";
    }
    //dbg(timers + "\nTotal time: " + (tmrs[tmrs.length-1].time - tmrs[0].time) + "ms");    
}


  function OnSelectionChange(){
    //if (getVal("ch_calc_fly")) calc_fast();
  }

/*
  function RegisterEvent()
  {
    // tell coreldraw to call OnSelectionChange when the selection changes
    window.external.RegisterEventListener( "SelectionChange", "OnSelectionChange()" );
  }

  function UnregisterEvent()
  {
    // if you want to stop listening for the selection change event, call this function
    window.external.UnregisterEventListener( "SelectionChange" );
  }

  function OnSelectionChange()
  {
    var objectName = "";
    if( window.external.Application.ActiveShape != null )
    {
      objectName = window.external.Application.Activeshape.Evaluate("@type"); // use CQL to get the object type name
    }
    if( window.external.Application.ActiveSelection != null )
    {
      numSelected.value = window.external.Application.ActiveSelection.Shapes.Count;
    }
    selectedObject.value = objectName;
  }
*/



function no_doc(){
    divscroll.innerHTML = '<div align="center"><input type="button" id="btn_nodoc" value="' + lang['no_doc'] + '" onClick="location.reload();" /></div>';
}

function correct_body_size(){
    var bodyRect = document.body.getBoundingClientRect();
    if(divscroll){
        var tRect = all_tabs.getBoundingClientRect();
        if (tRect.bottom<200){
            var bRect = div_copy.getBoundingClientRect();
            var newHeight = (bodyRect.bottom - bRect.bottom + bRect.top - tRect.bottom + tRect.top-13);
            if (newHeight<100) newHeight = 100;
            divscroll.style.height =  newHeight;
        }
    }
}
//window.onload = correct_body_size;
window.onresize = correct_body_size;

// show_title теперь реализован в ModuleSystem.createModuleStructure()

// cat_state теперь должна быть реализована в moduleSystem.js

function showhide(id, save){
    if(id == "warning") glSelect = undefined;
	if(save === undefined) save = false;
	var cat = document.getElementById(id);
	cat.style.display = (cat.style.display != "block") ? "block" : "none";
	if (save) do_save(cat);
}

// ShowCat теперь переопределяется в moduleSystem.js

// Заменить один класс на другой
function replaceClass(element, oldClass, newClass) {
    var classes = element.className.split(" ");
    for(var i = 0; i < classes.length; i++) {
        if(classes[i] == oldClass) {
            classes[i] = newClass;
        }
    }
    element.className = classes.join(" ");
}


// do_move теперь реализован как ModuleSystem.moveModule()

function do_events(bid){
	//warn("");
}

function do_save(el,min,max){
	if(min === undefined) min = false;
	if(max === undefined) max = false;
    if (window.event) window.event.cancelBubble = true;
	var key = el.id;
	var val = null;
    //if(key=="ch_cont_del") if (getVal(el))  ch_cont_sep.checked = true;

	if (key.substring(0,4)=="cat_"){
		val = (el.style.display == "block") ? true : false;
	}else if(el.type=="checkbox"){
		val = getVal(el);
        // ✅ Убрана принудительная установка чекбокса настроек в true
        active_deactive(el);
	}else{
		val = el.value;
	}

	if(el.type=="radio"){
		var elems = document.getElementsByName(el.name);
		for (var i = 0; i < elems.length; i++){
			val = (getVal(elems[i])) ? "true" : "false";
			vbWrite(elems[i].id,val);
            set_vals(el.name, el.value, val);
            if (el.id==elems[i].id) active_deactive(elems[i]);
		}
	}else if(val!=null){
		if (min && max){
			val = vall(val,  false);
			if (val<min) val=min;
			if (val>max) val=max;
		}
		if(val === true) val = "true";
		if(val === false) val = "false";
		vbWrite(key,val);
	}

    //if (el.id=="ch_cont_names") contur_names.style.display = (getVal(ch_cont_names)) ? "block" : "none";
    //if (el.id=="val_kbx_th") if (!getVal(ch_kbx_close_gap)) val_kbx_close_gap.value = el.value;
    if(inArray(key, ["bgcolor", "ch_set_titles_move", "ch_set_titles", "val_set_conturs", "val_add_lists", "css_style", "lang"])){
        location.reload();
    }
    // load_gbx() теперь вызывается внутри модуля cat_kbx
}

// do_read() функция удалена - её логика встроена в ModuleSystem.initializeModuleUI()

function select_color(ids, solo){
    if(solo === undefined) solo = false;

    //do_events(ids.id);
    var instr = ids.name.indexOf("_sel");
    var idstart = ids.id.substr(0,7);
    var adds = ids.name;
    if(instr>=0){
        if (idstart=='ffcolor') ids.innerHTML = ids.step;
        else if (inArray(idstart,['ocolor_','fcolor_']) && ids.className!="color_btn3") ids.value = ids.step;
        ids.style.backgroundImage= "";
        ids.name=ids.name.substr(0,instr);
    }else{
        if (idstart=='ffcolor') ids.innerHTML = "";
        else if (inArray(idstart,['ocolor_','fcolor_']) && ids.className!="color_btn3") ids.value = "";
        ids.style.backgroundImage= "url('img/check.png')";
        ids.name=ids.name + "_sel";
    }
    if (Shift()==4 && !solo){
        var elems = document.getElementsByTagName('*');
        for (var i = 0; i < elems.length; i++){
            if(elems[i].name==adds) select_color(elems[i], true);
        }
        Shift(true);
    }
}



function get_hint(key){
    var tit = "";
    if(!hint[key]){
        //alert(key);
    }else if(hint[key][0]){
        var tit = (hint[key][0][0]) ? tit = hint[key][0][0] : "";
        tit += "\n";
        for(var shft in hint[key]){
            if (!hint[key][shft]) alert(key + ":" +shft);
            if (shft>0 && hint[key][shft][0]){
                var skey = (glSkeys[shft]) ? " + " + glSkeys[shft] + " - " : "";
                tit += "\n" + skey + hint[key][shft][0];
            }
        }
    }else if(hint[key]){
        var tit = hint[key];
    }
    return tit;
}



function show_help(el, help, code){
    if(help === undefined) help = false;
    if(code === undefined) code = "";
	var elid = el.id;
    var tit = "";
    var cont = false;
    if (elid.substr(0,5)=="cont_"){
        cont = elid.substr(elid.length-1);
        elid = elid.substr(0,elid.length-1);
    }
    if(Shift()==0 && !help && hint[elid]) if(hint[elid][0] || code!=""){
        var viewportOffset = el.getBoundingClientRect();
        // these are relative to the viewport, i.e. the window
        var top = viewportOffset.top;
        var left = viewportOffset.left;
        var bodyRect = document.body.getBoundingClientRect();
        var body_c = (bodyRect.right - bodyRect.left)/2;
        var elRect = el.getBoundingClientRect();
        var scrll = document.body.scrollTop;

        if (elRect.right>=body_c){
            btns_adds.style.textAlign="right";
            btns_adds.style.left = "10%";
            btns_adds.style.backgroundPosition="top left";
        }
        if (elRect.right<body_c){
            btns_adds.style.textAlign="left";
            btns_adds.style.left = 5;
            btns_adds.style.backgroundPosition="top right";
        }
        if (code==""){
            for(var shft in hint[elid]){
                if (hint[elid][0][2]){
                    var skey = (glSkeys[shft]) ? " " + glSkeys[shft] + "" : "";
                    var func = (hint[elid][shft][2]) ? hint[elid][shft][2] : hint[elid][0][2];
                    if (cont) func = replaceAll(func,"{i}",cont);
                    var val = (hint[elid][shft][1]) ? hint[elid][shft][1] : hint[elid][shft][0];
                    if (elRect.right>=body_c) tit += skey;
                    tit += '<input type="button" onMouseUp="MouseUP('+el.id+', true);" class="btn_adds" onclick="hide_me(this,'+shft+'); '+func+';Shift(1);" value="'+val+'" title="'+hint[elid][shft][0]+'" />';
                    if (elRect.right<body_c) tit += skey;
                    tit +="<br />";
                }
            }
            if (hint[elid][0][0]) tit += '<p class=hint_popup>'+hint[elid][0][0]+' <a href="#" onCLick="show_help('+el.id+',1)">'+lang['text_more_help']+'</a></p>'
        }else{
            tit = '<p class=hint_popup>'+code+'</p>';
        }
    }
    if (tit.length>10){
        btns_adds.style.display = "block";
        btns_adds.innerHTML = tit;
        var eltop = elRect.top+scrll;
//        dbg("tit.length: " + tit.length + " html:" + btns_adds.innerHTML);
        btns_adds.style.top = eltop+15;
        var rect = btns_adds.getBoundingClientRect();
        if (rect.bottom>bodyRect.bottom){
            btns_adds.style.top = elRect.bottom-(rect.bottom-rect.top)+scrll-17;
        }
    }else{
        if (elid.substr(0,4)=="img_") elid = elid.substr(4);
        if (hint[elid]){
            var hints = get_hint(elid);
            var adds = CR + CR + CR;
        }else{
            var hints = "";
            var adds = "";
        }

        var descr = (desc[elid]) ? hints + adds + desc[elid] : hints;
        //alert(elid + "=" + descr.length + ":" + hint + "----" + descr);
        descr += "\n\nID: " + el.id;
        if (descr.length>3){
            if (utub[elid]){
                if(confirm(descr + CR + CR + CR + lang['text_youtube'])){
                    //window.open('https://youtu.be/' + utub[elid]);
                    vbRun(utub[elid], undefined);
                }
            }else{
                alert(descr);
            }
        }
    }

	return false;
}
function isSafeClick(src) {
    return (src.tagName === "INPUT" &&  (src.name === "variant" || src.id === "nest_fill"));
}

function hide_me(el, shft){
    if(shft === undefined) shft = 99;
    glShift = shft;
    //dbg(el.tagName);
    //dbg(window.event.srcElement.tagName);
    //dbg(window.event.srcElement.id);
    //dbg(window.event.srcElement.name);
    if (!el || !isSafeClick(window.event.srcElement)) btns_adds.style.display = "none";
}


function active_deactive(el, recursive, mode){   

    if (recursive === undefined) recursive = true;
    if (mode === undefined) mode = "normal";
    var rad = null;
    if (el.type == "radio"){
        rad = (el.value == getVal(el.name)) ? true : false;
    }   
    if (el.active && (rad===true || rad===null)){
        var act = el.active.split(",");
        for (var i = 0; i < act.length; i++){
            var lnk = document.getElementById(act[i]);
            if(lnk){
                if (!(mode=="read" && lnk.disabled && (lnk.type == "radio" || lnk.type == "checkbox"))) lnk.disabled = !getVal(el);
                if (recursive) active_deactive(lnk, false,mode);
            }
        }
    }
    if(el.deactive && (rad===true || rad===null)){
        var deact = el.deactive.split(",");
        for (var i = 0; i < deact.length; i++){
            var lnk = document.getElementById(deact[i]);
            if(lnk){
                if (!(mode=="read" && lnk.disabled && (lnk.type == "radio" || lnk.type == "checkbox"))) lnk.disabled = getVal(el);
                if (recursive) active_deactive(lnk, false,mode);
            }
        }
    }
    if (el.checkON && el.checked && (rad===true || rad===null)){
        var act = el.checkON.split(",");
        for (var i = 0; i < act.length; i++){
            var lnk = document.getElementById(act[i]);
            if(lnk){
                lnk.checked = true;
                if (recursive) active_deactive(lnk, false,mode);
            }
        }
    }

    if (el.uncheckON && el.checked && (rad===true || rad===null)){
        var act = el.uncheckON.split(",");
        for (var i = 0; i < act.length; i++){
            var lnk = document.getElementById(act[i]);
            if(lnk){
                lnk.checked = false;
                if (recursive) active_deactive(lnk, false,mode);
            }
        }
    }
    
    if (el.checkOFF && !el.checked && (rad===true || rad===null)){
        var act = el.checkOFF.split(",");
        for (var i = 0; i < act.length; i++){
            var lnk = document.getElementById(act[i]);
            if(lnk){
                lnk.checked = true;
                if (recursive) active_deactive(lnk, false,mode);
            }
        }
    }

    if (el.uncheckOFF && !el.checked && (rad===true || rad===null)){
        var act = el.uncheckOFF.split(",");
        for (var i = 0; i < act.length; i++){
            var lnk = document.getElementById(act[i]);
            if(lnk){
                lnk.checked = false;
                if (recursive) active_deactive(lnk, false,mode);
            }
        }
    }

    if (el.show){
        var act = el.show.split(",");
        for (var i = 0; i < act.length; i++){
            var lnk = document.getElementById(act[i]);
            if(lnk){
                lnk.style.display = el.checked ? "block" : "none";
            }
        }
    }
    
}

/*
function getRadio(name) {
    var elems = document.getElementsByName(name);
    for (var i = 0; i < elems.length; i++) {
        if (elems[i].disabled) return false;
        if (elems[i].checked) return elems[i].value;
    }
    return false;
}

function getCheck(checkbox) {
    if(typeof checkbox === "string") var checkbox = document.getElementById(checkbox);
        if (checkbox) return !checkbox.disabled && checkbox.checked;
    return false;
}
*/

/**
 * Установка значения элемента формы
 * @param {string|HTMLElement} elem - ID элемента или сам элемент
 * @param {*} val - значение для установки
 * @param {boolean} [save=true] - сохранять ли значение в настройках
 * 
 * Поддерживает:
 * - Радио-кнопки (по имени группы)
 * - Чекбоксы
 * - Текстовые поля
 * - Селекты
 */
function setVal(elem, val, save){
    if (save === undefined) save = true;
    var element;

    if (typeof elem === "string") {
        element = document.getElementById(elem);
        if (!element) {
            var elements = document.getElementsByName(elem);
            if (elements.length > 0) {
                // Для радио-кнопок нам нужна вся группа, для остального - первый элемент
                element = (elements.length > 1 && elements[0].type !== 'radio') ? elements[0] : elements;
            }
        }
    } else {
        element = elem;
    }

    if (!element) return;

    var firstEl = element.length ? element[0] : element;

    if (firstEl.type === "radio"){
        if (typeof val === "boolean") {
            // Если передано boolean значение, устанавливаем конкретную кнопку
            element.checked = val;
            if (val) {
                // Если устанавливаем в true, снимаем остальные в группе
                var radioGroup = document.getElementsByName(firstEl.name);
                for (var i = 0; i < radioGroup.length; i++)  {
                    if (radioGroup[i].id !== element.id) {
                        radioGroup[i].checked = false;
                    }
                }
            }
        } else {
            // Если передано строковое значение, работаем как раньше
            var radioGroup = document.getElementsByName(firstEl.name);
            for (var i = 0; i < radioGroup.length; i++)  {
                radioGroup[i].checked = (radioGroup[i].value == val);
            }
        }
    } else if (element.type=="checkbox") {
        element.checked = val;
    } else {
        element.value = val;
    }

    if (save) do_save(element);
}

/**
 * Получение значения элемента формы
 * @param {string|HTMLElement} elem - ID элемента, имя группы радио-кнопок или сам элемент
 * @param {boolean} [numb=false] - преобразовывать ли значение в число
 * @param {boolean} [scale=true] - применять ли масштабирование к числу
 * @returns {*} значение элемента
 * 
 * Поддерживает:
 * - Радио-кнопки (возвращает значение выбранной кнопки)
 * - Чекбоксы (возвращает true/false)
 * - Текстовые поля (возвращает значение или число)
 * - Селекты (возвращает выбранное значение)
 */
function getVal(elem, numb, scale) {
    if (numb === undefined) numb = false;
    if (scale === undefined) scale = true;
    var byName = [];
    // если elem — строка, получаем элемент по ID или Name

    //if (elem.id == "gbx_list") alert(elem.type);


    if (elem.length>0 && elem.type!=="select-one"){
        // пробуем найти по ID
        var el = document.getElementById(elem);
        // если не найден, ищем по имени
        if (!el || elem != el.id) {
            byName = (typeof elem === "object") ? elem : document.getElementsByName(elem);
            if (byName.length > 0) el = byName[0];
        }
        if (el) elem = el;
    }
    if (!elem) return false;

    // RADIO GROUP
    if (elem.type === "radio" && byName.length > 0) {
        var radios = document.getElementsByName(elem.name);
        for (var i = 0; i < radios.length; i++) {
            if (radios[i].disabled) continue;
            if (radios[i].checked) return radios[i].value;
        }
        return false;
    }else if (elem.type === "radio"  || elem.type === "checkbox") {
        return !elem.disabled && elem.checked;
    }

    // OTHER INPUTS
    if (elem.disabled) return 0;
    var val = elem.value;
    return (numb) ? vall(val, scale) : val;
}

function MouseUP(el, help){
    // ✅ Статистика нажатий кнопок
    if (el.id=="debug" && window.event.button==4){
         el.innerHTML = "";
         return false;
    }
    if (window.event.button == 1) setStat(el.id);

    if (window.event && window.event.button == 4) { // Middle mouse button ONLY
        var id = el.id;
        var moduleName = null;

        if (id.substring(0, 10) === 'title_cat_' || id.substring(0, 11) === 'tit_cat_') {
            moduleName = id.substring(id.indexOf('cat_'));
        } else if (id.substring(0, 10) === 'img_title_') {
            moduleName = id.substring(10);
        }
        
        if (moduleName && typeof ModuleSystem !== 'undefined' && ModuleSystem.isolateModule) {
            ModuleSystem.isolateModule(moduleName);
        } else {
            ModuleSystem.showHelp(el, true); // Show detailed help for other elements
        }
        return;
    }

    if(help===undefined) help = false;
    if (isSafeClick(window.event.srcElement)) return false;

    if (event.button == 2){
        ModuleSystem.showHelp(el, help);
    }
    if (event.button == 4){
        if (el.id=="debug") 
            dbg(); 
        else 
            if (el.tag) ShowCat(el.tag); else ModuleSystem.showHelp(el, 1);
    }
    return false;
}

function setStat(id){
    if (window.event.button == 1){
        var rd = vbRead('btn_s_' + id);
        if (rd) rd++; else rd = 1;
        vbWrite('btn_s_' + id, rd);
    }
}

function viewStat(){
    glDbg = true;
    var stat_btn = [];
    var stat_cat = [];
    var elems = document.getElementsByTagName('*');
    for (var i = 0; i < elems.length; i++){
        var el = elems[i];
        if (!el.id) continue;
        var rd = vbRead('btn_s_' + el.id);
        if (rd){
            var tit = btns[el.id] ? btns[el.id] : (lang[el.id] ? lang[el.id] : el.id);
            var hnt = (hint[el.id]) ? (typeof hint[el.id] == "string" ? hint[el.id] : hint[el.id][0][0]) : "";
            if (el.id.substr(0,8)=="tit_cat_")
                stat_cat.push({id:el.id, tit:tit, hint:hnt, cnt:rd});
            else
                stat_btn.push({id:el.id, tit:tit, hint:hnt, cnt:rd});
        }
    }
    stat_btn.sort(function(a, b) { return b.cnt - a.cnt; });
    dbg("<br />stat_btn: "+stat_btn.length);
    for (var i = 0; i < stat_btn.length; i++){
        dbg('<span title="' + stat_btn[i].hint + '">' +  stat_btn[i].tit + "=" + stat_btn[i].cnt + "</span>");
    }
    dbg("<br />stat_cat: "+stat_cat.length);
    stat_cat.sort(function(a, b) { return b.cnt - a.cnt; });
    for (var i = 0; i < stat_cat.length; i++){
        dbg('<span title="' + stat_cat[i].hint + '">' +  stat_cat[i].tit + "=" + stat_cat[i].cnt + "</span>");
    }
    glDbg = false;
}

document.oncontextmenu = function() {
  return false; // Предотвращаем появление контекстного меню
}

function show_copy(){
    if (Shift() == 4) {viewStat(); return;}
    if(confirm(lang['copy'] + CR + CR + CR + lang['text_youtube'])){
        vbRun(utub["install"], undefined);
    }
}

setInterval(function(){
    var btn_id = vbRead("docker_btn");
    if (btn_id===undefined) return;
    var btn = document.getElementById(btn_id);
    if (!btn){
        var tool = vbRead("docker_tool");
        if (tool!==undefined) ModuleSystem.loadModule(tool);
        btn = document.getElementById(btn_id);
    }

    if (btn){
        var shft = vbRead("docker_shift");
        if (shft!==undefined) glShift = shft;
        //dbg("docker_btn: "+btn.id+" glShift: "+glShift);
        btn.click();
    }else{
        //dbg("docker_btn: "+btn_id+" not found");
    }
    vbWrite("docker_btn", "");
    vbWrite("docker_shift", "");
    vbWrite("docker_tool", "");
}, 1000); // проверяем каждую секунду





tmrs.push({name:"htmlsys end",time:new Date().getTime()});