// Модульная система wpanel - динамическая загрузка инструментов
//
// ЗАВИСИМОСТИ: требует htmlsys.js (использует функцию getElementsByClassName)

// ✅ Глобальный реестр функций инициализации модулей
var ModuleInitRegistry = {};

var ModuleSystem = {
    toolsPath: "tools/",
    loadedModules: {},
    availableModules: [],
    
    // Кэшированные настройки интерфейса
    showTitles: false,
    moveMode: false,
    
    // Обновление кэшированных настроек интерфейса
    updateInterfaceSettings: function() {
        this.showTitles = vbRead('ch_set_titles')===true ? true : false;
        this.moveMode = vbRead('ch_set_titles_move')===true ? true : false;
    },
    
    // Инициализация системы при загрузке страницы
    init: function() {
        this.updateInterfaceSettings();
        this.scanToolsDirectory();
        this.generateModuleStructures();
        this.replaceShowCat();
        this.restoreAllModuleStates();
       
        // 🎨 Применяем чередование цветов при первой загрузке
        this.updateBodyOddEvenClasses();
        // Инициализация глобальных стилей для изображений
        this.initStyle();
    },
    
    initStyle: function() {
        if (glStyle!=""){
            var elems = document.getElementsByTagName('img');
            for (var i = 0; i < elems.length; i++){
                var el = elems[i];
                var newsrc = el.src.replace("img/", "img/"+glStyle);
                if(vbfileExist(newsrc)) el.src = newsrc;
            }
        } 
    },

    // Динамическое сканирование папки tools
    scanToolsDirectory: function() {
        try {
            // Получаем список HTML файлов из папки tools
            var htmlFiles = vbReadFolder('tools', '*.html');
            
            if (htmlFiles && htmlFiles.length > 0) {
                for (var i = 0; i < htmlFiles.length; i++) {
                    var fileName = htmlFiles[i];
                    
                    // Убираем расширение .html чтобы получить имя модуля
                    var moduleName = fileName.replace(/\.html$/i, '');
                    
                    if (vbRead('ch_show_title_'+moduleName)===false && !this.moveMode) continue;

                    // Добавляем в список доступных модулей
                    this.availableModules.push(moduleName);
                }
                
            }
            
        } catch (e) {
            dbg("ModuleSystem: ошибка сканирования папки tools - " + e.message);
        }
    },
    
    // Создание структур для всех модулей
    generateModuleStructures: function() {
        var container = document.getElementById('divscroll');
        if (!container) return;
        
        var orderedModules = this.getOrderedVisibleModules();
        
        for (var i = 0; i < orderedModules.length; i++) {
            var moduleName = orderedModules[i];
            
            // Пропускаем уже существующие модули
            if (document.getElementById('title_' + moduleName)) continue;
            
            this.createModuleStructure(container, moduleName);
            this.createModuleButton(moduleName);
        }
        
        // После создания всех элементов перемещаем debug в конец
        this.pinDebugElement();
        // Чередование классов
        if (typeof ModuleSystem.updateBodyOddEvenClasses === 'function') ModuleSystem.updateBodyOddEvenClasses();
    },
    
    // Получение упорядоченного списка видимых модулей  
    getOrderedVisibleModules: function() {
        var result = [];
        var isMoveMode = this.moveMode;
        
        // Получаем пользовательский порядок из настроек
        try {
            var catsOrder = vbRead('cats');
            if (catsOrder) {
                var userOrder = catsOrder.split('/');
                
                // Добавляем модули в пользовательском порядке (исключая cat_settings)
                for (var i = 0; i < userOrder.length; i++) {
                    var titleId = userOrder[i];
                    if (titleId && titleId.substring(0, 6) === 'title_') {
                        var moduleName = titleId.substring(6);
                        
                        // cat_settings обрабатываем отдельно
                        if (moduleName === 'cat_settings') continue;
                        
                        var isVisible = vbRead('ch_show_' + titleId)===false ? false : true;
                        if (inArray(moduleName, this.availableModules) && 
                            (isMoveMode || isVisible)) {
                            result.push(moduleName);
                        }
                    }
                }
            }
        } catch (e) {}
        
        // Добавляем новые модули не из порядка (исключая cat_settings)
        for (var j = 0; j < this.availableModules.length; j++) {
            var moduleName = this.availableModules[j];
            
            // cat_settings обрабатываем отдельно
            if (moduleName === 'cat_settings') continue;
            
            var isVisible = vbRead('ch_show_title_' + moduleName)===false ? false : true;
            if (!inArray(moduleName, result) && 
                (isMoveMode || isVisible)) {
                result.push(moduleName);
            }
        }
        
        // 🔧 cat_settings всегда добавляем в конец и всегда видимый
        if (inArray('cat_settings', this.availableModules)) {
            result.push('cat_settings');
        }
        
        return result;
    },
    
    // Получение локализованного заголовка модуля
    getModuleTitle: function(moduleName) {
        var titleKey = 'title_' + moduleName;
        var title = '';
        
        if (lang && lang[titleKey]) {
            title = lang[titleKey];
        } else {
            // Заменяем префикс и все подчеркивания на пробелы
            title = moduleName.replace('cat_', '').replace(/_/g, ' ');
        }
        
        // 🔧 Для cat_settings добавляем индикацию что это защищенный модуль
        if (moduleName === 'cat_settings') {
            title = '[*] ' + title; // Индикация защищенного модуля
        }
        
        return title;
    },
    
    // Создание структуры заголовка и тела модуля
    createModuleStructure: function(container, moduleName) {
        try {
            // Создаем заголовок
            var titleDiv = document.createElement('div');
            titleDiv.className = 'cat_title';
            titleDiv.id = 'title_' + moduleName;
            titleDiv.onmouseup = function() { MouseUP(this); };
            titleDiv.onclick = (function(name) {
                return function() { ShowCat(name); };
            })(moduleName);
            titleDiv.innerHTML = this.getModuleTitle(moduleName);
            // Если включен режим перемещения, добавляем стрелки и чекбокс
            // 🔧 Для cat_settings элементы управления не показываем (нельзя двигать и скрывать)
            if (this.moveMode && moduleName !== 'cat_settings') {
                float_state_mem_btns.style.display = 'none';
                // Чекбокс видимости (самый правый)
                var showCheckSpan = document.createElement('span');
                showCheckSpan.className = 'move_arrow';
                showCheckSpan.style.position = 'absolute';
                showCheckSpan.style.right = '5px';
                showCheckSpan.style.top = '5%';
                showCheckSpan.style.transform = 'translateY(-50%)';
                var checkbox = document.createElement('input');
                checkbox.type = 'checkbox';
                checkbox.className = 'noborder';
                checkbox.id = 'ch_show_' + titleDiv.id;
                checkbox.checked = true;
                checkbox.onclick = function(event) {
                    window.event.cancelBubble = true;
                    do_save(this);
                };
                showCheckSpan.appendChild(checkbox);
                var isChecked = vbRead(checkbox.id)===false ? false : true;
                setVal(checkbox, isChecked);

                // Стрелка вниз
                var moveDown = document.createElement('span');
                moveDown.className = 'move_arrow';
                moveDown.style.position = 'absolute';
                moveDown.style.right = '30px';
                moveDown.style.top = '5%';
                moveDown.style.transform = 'translateY(-50%)';
                moveDown.style.cursor = 'pointer';
                moveDown.innerHTML = '&#9660;'; // Down arrow
                moveDown.onclick = function(event) {
                    window.event.cancelBubble = true;
                    ModuleSystem.moveModule(moduleName, 'down');
                };

                // Стрелка вверх
                var moveUp = document.createElement('span');
                moveUp.className = 'move_arrow';
                moveUp.style.position = 'absolute';
                moveUp.style.right = '50px';
                moveUp.style.top = '5%';
                moveUp.style.transform = 'translateY(-50%)';
                moveUp.style.cursor = 'pointer';
                moveUp.innerHTML = '&#9650;'; // Up arrow
                moveUp.onclick = function(event) {
                    window.event.cancelBubble = true;
                    ModuleSystem.moveModule(moduleName, 'up');
                };

                // Добавляем элементы управления к заголовку
                titleDiv.style.position = 'relative'; // Для абсолютного позиционирования детей
                titleDiv.style.minHeight = '24px'; // Увеличиваем высоту для элементов управления
                titleDiv.style.lineHeight = '24px'; // Центрируем текст по вертикали
                titleDiv.style.paddingRight = '40px'; // Отступ справа для элементов управления
                titleDiv.appendChild(showCheckSpan);
                titleDiv.appendChild(moveDown);
                titleDiv.appendChild(moveUp);
            }
            
            // 🔥 СНАЧАЛА добавляем заголовок 
            container.appendChild(titleDiv);
            
            // 🔥 ПОТОМ создаем и добавляем тело модуля (под заголовком)
            var bodyDiv = document.createElement('div');
            bodyDiv.id = moduleName;
            bodyDiv.className = 'cat_body';
            bodyDiv.style.display = 'none';
            container.appendChild(bodyDiv);

            // show_title больше не существует - состояние заголовка управляется через applyShowCatLogic
            
        } catch (e) {
            dbg("ModuleSystem: ошибка создания " + moduleName + ": " + e.message);
        }
    },
    
    // Создание кнопки для модуля
    createModuleButton: function(moduleName) {
        // Проверяем нужна ли кнопка (когда заголовки выключены)
        if (this.showTitles || this.moveMode) {
            return;
        }
        
        var all_tabs = document.getElementById('all_tabs');
        if (!all_tabs) return;
        
        var titleId = 'title_' + moduleName;
        var title = this.getModuleTitle(moduleName);
        var tooltip = title;
        if (typeof hint !== 'undefined' && hint[titleId]) {
            tooltip += "\n\n" + hint[titleId];
        }
        
        var img = document.createElement('span');
        img.innerHTML = '<img src="img/' + titleId + '.png" onMouseUp="MouseUP(this);" tag="' + moduleName + '" id="img_' + titleId + '" class="float_but_title" onClick="ShowCat(\'' + moduleName + '\')" title="' + tooltip + '"/>';
        
        all_tabs.appendChild(img);
        
        // Дополнительно привязываем обработчик событий программно для надежности
        var createdButton = document.getElementById('img_' + titleId);
        if (createdButton) {
            createdButton.onclick = function() { 
                ShowCat(moduleName); 
            };
        }
        
        // Изначально неактивна
        var imgElement = document.getElementById('img_' + titleId);
        if (imgElement) {
            imgElement.style.filter = 'alpha(opacity=30)';
        }
    },
    
    // Восстановление состояний всех модулей при старте
    restoreAllModuleStates: function() {
        for (var i = 0; i < this.availableModules.length; i++) {
            var moduleName = this.availableModules[i];
            var bodyElement = document.getElementById(moduleName);
            var moduleState = vbRead(moduleName);
            
            if (moduleState && bodyElement) {
                // Показываем заглушку и загружаем контент
                bodyElement.style.display = 'block';
                bodyElement.innerHTML = '<div style="padding:10px;color:#666;font-style:italic;">Загрузка...</div>';
                this.updateButtonState(moduleName, true);
                this.loadModule(moduleName);
                // 🔥 КРИТИЧНО: Обновляем состояние заголовка!
                this.applyShowCatLogic(moduleName, true);
            } else {
                this.updateButtonState(moduleName, false);
                // 🔥 КРИТИЧНО: Обновляем состояние заголовка!
                this.applyShowCatLogic(moduleName, false);
            }
        }
        this.updateStateButtons();   
    },
    
    // Замена функции ShowCat новой логикой
    replaceShowCat: function() {
        var self = this; // Сохраняем контекст ModuleSystem

        ShowCat = function(cat_id, show) {
            // Проверяем, есть ли элемент в DOM
            var bodyElement = document.getElementById(cat_id);
            if (!bodyElement) {
                return;
            }
            
            // Проверяем нажатие Alt или среднюю кнопку мыши
            var isAltClick = isAlt();
            var isMiddleClick = window.event && window.event.button === 1;
            var shouldIsolate = isAltClick || isMiddleClick;
            
            // Если Alt или средняя кнопка - изолируем модуль
            if (shouldIsolate) {
                self.isolateModule(cat_id);
                return;
            }
            
            // Если это динамический модуль из папки tools
            var needsLoad = self.needsLoading(cat_id);
            
            if (needsLoad) {
                self.loadModule(cat_id, function() {
                    self.applyShowCatLogic(cat_id, show);
                });
            } else {
                self.applyShowCatLogic(cat_id, show);
            }
        };
    },
    
    // Логика отображения/скрытия, вынесенная для повторного использования
    applyShowCatLogic: function(moduleName, show) {
        try {
            var bodyElement = document.getElementById(moduleName);
            if (!bodyElement) {
                return;
            }
            
            var titleElement = document.getElementById('title_' + moduleName) || document.getElementById('tit_' + moduleName);
            var wasVisible = bodyElement.style.display !== 'none';
            var shouldShow = (show === undefined) ? !wasVisible : show;
            
            // Применяем изменения только если видимость меняется
            var showMainTitles = (this.showTitles || this.moveMode);
            if (wasVisible !== shouldShow || showMainTitles) {
                bodyElement.style.display = shouldShow ? 'block' : 'none';
                if (titleElement) {
                    var isInnerTool = titleElement.className.indexOf('cat_tit ') > -1;

                    if (isInnerTool) {
                        titleElement.style.display = 'block';
                        titleElement.className = shouldShow ? 'cat_tit cat_arrow_up' : 'cat_tit cat_arrow_dn';
                    } else {
                        if (showMainTitles) {
                            titleElement.style.display = 'block';
                            titleElement.className = 'cat_title';
                            if (showMainTitles) titleElement.className += shouldShow ? ' cat_arrow_up' : ' cat_arrow_dn';
                        } else {
                            titleElement.style.display = 'none';
                        }
                    }
                }
                this.updateButtonState(moduleName, shouldShow);
                vbWrite(moduleName, shouldShow ? "true" : "false");
                if (shouldShow) {
                    vbWrite('lastcat', moduleName);
                }
                if (typeof correct_body_size === 'function') {
                    try { correct_body_size(); } catch(e) {}
                }
                if (typeof ModuleSystem.updateStateButtons === 'function') ModuleSystem.updateStateButtons();
                if (typeof ModuleSystem.updateBodyOddEvenClasses === 'function') ModuleSystem.updateBodyOddEvenClasses();
            }
        } catch (e) {
            // Игнорируем ошибки отображения модуля
        }
    },
    
    // Проверка, нужна ли загрузка модуля
    needsLoading: function(moduleName) {
        return inArray(moduleName, this.availableModules) && (!this.loadedModules[moduleName] || !this.loadedModules[moduleName].loaded);
    },
    
    // Загрузка модуля
    loadModule: function(moduleName, callback) {
        var htmlPath = this.toolsPath + moduleName + '.html';
        
        // Сначала загружаем HTML. Если успешно:
        if (includeHTML(htmlPath, moduleName)) {
            var jsPath = this.toolsPath + moduleName + '.js';
            include(jsPath); // include сам обрабатывает ошибки
            
            // Проверяем наличие функции инициализации в реестре
            var moduleInitFunc = ModuleInitRegistry[moduleName] || null;
            
            // Сохраняем ссылку в объекте модуля
            if (!this.loadedModules[moduleName]) this.loadedModules[moduleName] = {};
            this.loadedModules[moduleName].initFunc = moduleInitFunc;
            
            // Инициализируем UI элементы в загруженном модуле
            this.initializeModuleUI(moduleName);

            // После вставки HTML может измениться высота – скорректируем
            if (typeof correct_body_size === 'function') {
                try { correct_body_size(); } catch(e) {}
            }

            // Отмечаем как загруженный
            this.loadedModules[moduleName].loaded = true;
        }
        
        // Вызываем callback в любом случае, чтобы не блокировать интерфейс
        if (callback) {
            try {
                callback();
            } catch(e) {
                // Игнорируем ошибки callback
            }
        }
    },
    
    // Обновление состояния кнопки
    updateButtonState: function(moduleName, isExpanded) {
        var imgElement = document.getElementById('img_title_' + moduleName);
        if (imgElement) {
            imgElement.style.filter = isExpanded ? 'alpha(opacity=100)' : 'alpha(opacity=30)';
        }
    },
    
    // Инициализация UI элементов в загруженном модуле
    initializeModuleUI: function(moduleName) {
        var moduleElement = document.getElementById(moduleName);
        if (!moduleElement) return;

        var elements = moduleElement.getElementsByTagName('*');
        for (var i = 0; i < elements.length; i++) {
            var el = elements[i];
            
            // 1. Восстанавливаем состояние контролов из настроек
            if (el.id) {
                var val = vbRead(el.id);
                if (val != undefined) {
                    if (el.type=="checkbox" || el.type == "radio") {
                        el.checked = val;
                        active_deactive(el, false, "read");
                    } else if (el.id.substring(0, 4) === "val_") {
                        el.value = val;
                    }
                }
            }

            // 2. Локализация на основе data-* атрибутов и ID
            if (typeof lang !== 'undefined') {
                // `data-lang` для основного текста (лейблы, спаны)
                var langKey = el.getAttribute('data-lang');
                if (langKey && lang[langKey]) {
                    el.innerHTML = lang[langKey];
                    if (hint[langKey]) el.title = hint[langKey];
                }

                // `value` для кнопок на основе ID элемента из объекта `btns` (как в OnLoad)
                if (el.id && typeof btns !== 'undefined' && btns[el.id] && el.tagName === 'INPUT' && (el.type === 'button' || el.type === 'submit')) {
                    if (btns[el.id].substring(0,4) === "src:") {
                        el.style.backgroundImage = "url('" + btns[el.id].substring(4) + "')";
                        el.value = "";
                    } else {
                        el.value = btns[el.id];
                    }
                }

                // `title` для подсказок на основе ID элемента (как в OnLoad)
                if (el.id && typeof get_hint === 'function' && typeof hint !== 'undefined' && hint[el.id]) {
                    el.title = get_hint(el.id);
                }

                // `title` для <label> на основе их `for` атрибута (как в OnLoad)
                if (el.tagName === 'LABEL' && el.htmlFor && typeof get_hint === 'function' && typeof hint !== 'undefined' && hint[el.htmlFor]) {
                    el.title = get_hint(el.htmlFor);
                }
            }

            // === ДОБАВИТЬ В КОНЕЦ ФАЙЛА ===
            if (el.className && el.className.indexOf('cat_tit') > -1 && el.id.indexOf('tit_') === 0) {
                var blockId = el.id.substring(4); // убираем 'tit_'
                var state = vbRead(blockId)!==true ? false : true;
                var body = document.getElementById(blockId);
                if (body) {
                    var isOpen = (state);
                    body.style.display = isOpen ? 'block' : 'none';
                    el.className = isOpen ? 'cat_tit cat_arrow_up' : 'cat_tit cat_arrow_dn';
                }
            }
        }
        
        // Универсальная модуль-специфичная инициализация
        // Вызываем сохраненную функцию инициализации модуля
        if (this.loadedModules[moduleName] && this.loadedModules[moduleName].initFunc) {
            try {
                this.loadedModules[moduleName].initFunc();
            } catch(e) {
                // Игнорируем ошибки инициализации
            }
        }
    },
    
    moveModule: function(moduleName, direction) {
        // 🔧 cat_settings нельзя перемещать - он всегда должен быть последним
        if (moduleName === 'cat_settings') {
            return;
        }
        
        var container = document.getElementById('divscroll');
        var titleEl = document.getElementById('title_' + moduleName);
        var bodyEl = document.getElementById(moduleName);
        if (!container || !titleEl || !bodyEl) return;

        var allTitles = getElementsByClassName(container, 'cat_title');
        var titlesArray = allTitles; // Уже массив, не NodeList
        var currentIndex = -1;
        
        for(var i = 0; i < titlesArray.length; i++) {
            if (titlesArray[i].id === titleEl.id) {
                currentIndex = i;
                break;
            }
        }

        if (direction === 'up' && currentIndex > 0) {
            var targetEl = titlesArray[currentIndex - 1];
            container.insertBefore(titleEl, targetEl);
            container.insertBefore(bodyEl, targetEl);
        } else if (direction === 'down' && currentIndex < titlesArray.length - 1) {
            var targetEl = titlesArray[currentIndex + 1];
            
            // 🔧 Нельзя перемещать модули ниже cat_settings
            var targetModuleName = targetEl.id.substring(6); // убираем "title_"
            if (targetModuleName === 'cat_settings') {
                return; // cat_settings должен оставаться последним
            }
            
            // Поскольку мы перемещаем два элемента (title и body), 
            // нам нужно вставить их перед *следующим* элементом после target
            // 🔧 Используем nextSibling вместо nextElementSibling для совместимости с JS 1.5
            function getNextElement(element) {
                var next = element.nextSibling;
                while (next && next.nodeType !== 1) {
                    next = next.nextSibling;
                }
                return next;
            }
            
            var nextSibling = getNextElement(targetEl);
            var nextBody = document.getElementById(targetEl.id.substring(6));
            if (nextBody) {
                nextSibling = getNextElement(nextBody);
            }

            container.insertBefore(titleEl, nextSibling);
            container.insertBefore(bodyEl, nextSibling);
        }
        
        // Сохраняем новый порядок (исключая cat_settings - он всегда последний)
        var newOrder = [];
        var updatedTitles = getElementsByClassName(container, 'cat_title');
        for (var j = 0; j < updatedTitles.length; j++) {
            var titleId = updatedTitles[j].id;
            var moduleName = titleId.substring(6); // убираем "title_"
            
            // 🔧 cat_settings не включаем в сохраняемый порядок
            if (moduleName !== 'cat_settings') {
                newOrder.push(titleId);
            }
        }
        vbWrite('cats', newOrder.join('/'));
        // Чередование классов
        if (typeof ModuleSystem.updateBodyOddEvenClasses === 'function') ModuleSystem.updateBodyOddEvenClasses();
    },
    
    isolateModule: function(moduleNameToIsolate) {
        // Сначала скрываем все модули
        for (var i = 0; i < this.availableModules.length; i++) {
            var moduleName = this.availableModules[i];
            // Используем applyShowCatLogic чтобы корректно обновить и UI и состояние
            this.applyShowCatLogic(moduleName, false);
        }
        
        var self = this;
        // Если модуль ещё не загружен, сначала загрузим его, затем покажем
        if (this.needsLoading(moduleNameToIsolate)) {
            this.loadModule(moduleNameToIsolate, function(){
                self.applyShowCatLogic(moduleNameToIsolate, true);
            });
        } else {
            // Модуль уже есть – просто показываем
            this.applyShowCatLogic(moduleNameToIsolate, true);
        }
    },
    
    pinDebugElement: function() {
        var container = document.getElementById('divscroll');
        if (!container) return;

        var debugDiv = document.getElementById('debug');
        
        if (debugDiv && debugDiv.parentNode === container) {
            container.removeChild(debugDiv);
            container.appendChild(debugDiv);
        } else if (debugDiv) {
            container.appendChild(debugDiv);
        }
    },
    
    // Функция состояний модулей (замена cat_state)
    manageState: function(state, shft) {
        if (shft === undefined) shft = (typeof Shift === 'function') ? Shift() : 0;
        var stval = document.getElementById('val_state_' + state);
        var states = [];
        
        if (stval) {
            if (shft == 1) {
                // Сохранение текущего состояния
                var conf = true;
                if (state > 0 && stval.value.length > 2) {
                    if (!confirm(lang['text_state_overwrite'])) conf = false;
                }
                if (conf) {
                    for (var i = 0; i < this.availableModules.length; i++) {
                        var moduleName = this.availableModules[i];
                        var bodyElement = document.getElementById(moduleName);
                        if (bodyElement && bodyElement.style.display !== "none") {
                            states.push(moduleName);
                        }
                    }
                    stval.value = states.join(":");
                    do_save(stval);
                    if (state > 0) this.manageState(state, 0);
                }
            } else if (shft == 2) {
                // Открыть настройки
                this.applyShowCatLogic("cat_settings", true);
            } else {
                // Восстановление состояния
                var state_last = document.getElementById('state_last');
                var last_val = state_last ? parseInt(state_last.value || 0) : 0;
                
                if (last_val == state) {
                    state_last.value = 0;
                } else {
                    if (last_val < 1) this.manageState("last", 1);
                    state_last.value = state;
                }
                
                if (stval.value.length > 0) {
                    var states = stval.value.split(":");
                    for (var j = 0; j < this.availableModules.length; j++) {
                        var moduleName = this.availableModules[j];
                        var shouldShow = inArray(moduleName, states);

                        if (this.needsLoading(moduleName)) {
                            this.loadModule(moduleName, function(){
                                self.applyShowCatLogic(moduleName, true);
                            });
                        } else {
                            // Модуль уже есть – просто показываем
                            this.applyShowCatLogic(moduleName, shouldShow);
                        }
                    }
                }
            }
        }
        // Добавить вызов обновления кнопок
        if (typeof ModuleSystem.updateStateButtons === 'function') ModuleSystem.updateStateButtons();
    },
    
    // Функция сброса порядка модулей
    resetModulesOrder: function() {
        if (confirm(lang['cats_reset_confirm'])) {
            vbWrite('cats', "1/2");
            
            // Сбрасываем видимость всех модулей (кроме cat_settings - он всегда видимый)
            for (var i = 0; i < this.availableModules.length; i++) {
                var moduleName = this.availableModules[i];
                
                // 🔧 cat_settings всегда видимый, его настройку не трогаем
                if (moduleName !== 'cat_settings') {
                    vbWrite('ch_show_title_' + moduleName, "true");
                }
            }
            
            location.reload();
        }
    },
    
    // Диагностика системы
    diagnose: function() {
        dbg("=== ДИАГНОСТИКА МОДУЛЬНОЙ СИСТЕМЫ ===");
        dbg("1. Проверка основных функций:");
        dbg("  - vbReadFolder: " + (typeof vbReadFolder !== 'undefined' ? "✓" : "✗"));
        dbg("  - includeHTML: " + (typeof includeHTML !== 'undefined' ? "✓" : "✗"));
        dbg("  - include: " + (typeof include !== 'undefined' ? "✓" : "✗"));
        dbg("  - ShowCat: " + (typeof ShowCat !== 'undefined' ? "✓" : "✗"));
        dbg("  - active_deactive: " + (typeof active_deactive !== 'undefined' ? "✓" : "✗"));
        
        dbg("2. Проверка папки tools:");
        try {
            var files = vbReadFolder('tools', '*.html');
            if (files && files.length > 0) {
                dbg("  Найдено HTML файлов: " + files.length);
                for (var i = 0; i < files.length; i++) {
                    dbg("    - " + files[i]);
                }
            } else {
                dbg("  ✗ HTML файлы не найдены");
            }
        } catch (e) {
            dbg("  ✗ Ошибка: " + e.message);
        }
        
        dbg("3. Доступные модули:");
        for (var j = 0; j < this.availableModules.length; j++) {
            var moduleName = this.availableModules[j];
            var titleExists = document.getElementById('title_' + moduleName) ? "✓" : "✗";
            var bodyExists = document.getElementById(moduleName) ? "✓" : "✗";
            var loaded = (this.loadedModules[moduleName] && this.loadedModules[moduleName].loaded) ? "✓" : "○";
            var visible = !vbRead('ch_show_title_' + moduleName) ? "✓" : "✗";
            dbg("  " + moduleName + " [title:" + titleExists + " body:" + bodyExists + " loaded:" + loaded + " visible:" + visible + "]");
        }
        
        dbg("4. Настройки интерфейса:");
        var showTitles = this.showTitles;
        var moveMode = this.moveMode;
        dbg("  - Показывать заголовки: " + (showTitles ? "✓" : "✗"));
        dbg("  - Режим перемещения: " + (moveMode ? "✓" : "✗"));
        dbg("  - Режим отображения: " + (showTitles || moveMode ? "заголовки" : "кнопки"));
        
        dbg("=== КОНЕЦ ДИАГНОСТИКИ ===");
    },

    // Показ контекстной помощи и меню
    showHelp: function(el, help, code) {
        if (help === undefined) help = false;
        if (code === undefined) code = "";
        
        var elid = el.id;
        var tit = "";
        var cont = false;
        
        // Проверка на контурный элемент
        if (elid.substr(0,5) == "cont_") {
            cont = elid.substr(elid.length-1);
            elid = elid.substr(0,elid.length-1);
        }
        
        // Если нет модификаторов и есть подсказка
        if (Shift() == 0 && !help && hint[elid]) {
            if (hint[elid][0] || code != "") {
                var viewportOffset = el.getBoundingClientRect();
                var top = viewportOffset.top;
                var left = viewportOffset.left;
                var bodyRect = document.body.getBoundingClientRect();
                var body_c = (bodyRect.right - bodyRect.left)/2;
                var elRect = el.getBoundingClientRect();
                var scrll = document.body.scrollTop;

                // Позиционирование меню
                if (elRect.right >= body_c) {
                    btns_adds.style.textAlign = "right";
                    btns_adds.style.left = "10%";
                    btns_adds.style.backgroundPosition = "top left";
                }
                if (elRect.right < body_c) {
                    btns_adds.style.textAlign = "left";
                    btns_adds.style.left = "5px";
                    btns_adds.style.backgroundPosition = "top right";
                }

                // Генерация содержимого меню
                if (code == "") {
                    for (var shft in hint[elid]) {
                        if (hint[elid][0][2]) {
                            var skey = (glSkeys[shft]) ? " " + glSkeys[shft] + "" : "";
                            var func = (hint[elid][shft][2]) ? hint[elid][shft][2] : hint[elid][0][2];
                            if (cont) func = replaceAll(func,"{i}",cont);
                            var val = (hint[elid][shft][1]) ? hint[elid][shft][1] : hint[elid][shft][0];
                            
                            if (elRect.right >= body_c) tit += skey;
                            tit += '<input type="button" onMouseUp="ModuleSystem.handleMouseUp('+el.id+', true);" class="btn_adds" onclick="ModuleSystem.hideMenu(this,'+shft+'); '+func+'" value="'+val+'" title="'+hint[elid][shft][0]+'" />';
                            if (elRect.right < body_c) tit += skey;
                            tit += "<br />";
                        }
                    }
                    if (hint[elid][0][0]) {
                        tit += '<p class="hint_popup">'+hint[elid][0][0]+' <a href="#" onClick="ModuleSystem.showHelp('+el.id+',1)">'+lang['text_more_help']+'</a></p>';
                    }
                } else {
                    tit = '<p class="hint_popup">'+code+'</p>';
                }
            }
        }

        // Отображение меню или подсказки
        if (tit.length > 10) {
            btns_adds.style.display = "block";
            btns_adds.innerHTML = tit;
            var eltop = elRect.top + scrll;
            btns_adds.style.top = eltop + 15 + "px";
            
            var rect = btns_adds.getBoundingClientRect();
            if (rect.bottom > bodyRect.bottom) {
                btns_adds.style.top = (elRect.bottom-(rect.bottom-rect.top)+scrll-17) + "px";
            }
        } else {
            // Показ обычной подсказки
            if (elid.substr(0,4) == "img_") elid = elid.substr(4);
            
            var hints = hint[elid] ? this.getHint(elid) : "";
            var adds = hints ? "\n\n\n" : "";
            var descr = (desc[elid]) ? hints + adds + desc[elid] : hints;
            
            descr += "\n\nID: " + el.id;
            
            if (descr.length > 3) {
                if (utub[elid]) {
                    if(confirm(descr + "\n\n\n" + lang['text_youtube'])) {
                        vbRun(utub[elid], undefined);
                    }
                } else {
                    alert(descr);
                }
            }
        }
        return false;
    },

    // Обработка клика мыши для элементов меню
    handleMouseUp: function(el, help) {
        if (window.event.button == 1) setStat(el.id);
        if (help === undefined) help = false;
        if (isSafeClick(window.event.srcElement)) return false;

        if (event.button == 2) {
            this.showHelp(el, help);
        }
        if (event.button == 4) {
            if (el.id == "debug") {
                dbg();
            } else {
                if (el.tag) ShowCat(el.tag); else this.showHelp(el, 1);
            }
        }
        return false;
    },

    // Скрытие меню
    hideMenu: function(el, shft) {
        if (shft === undefined) shft = 99;
        // Устанавливаем оба глобальных состояния
        glShift = shft;
        if (typeof glShift !== 'undefined') glShift = shft;
        
        if (!el || !isSafeClick(window.event.srcElement)) {
            btns_adds.style.display = "none";
            // Сбрасываем состояния после выполнения функции
            setTimeout(function() {
                glShift = 99;
                if (typeof glShift !== 'undefined') glShift = 99;
            }, 100);
        }
    },

    // Получение текста подсказки
    getHint: function(key) {
        var tit = "";
        if (!hint[key]) {
            //alert(key);
        } else if (hint[key][0]) {
            var tit = (hint[key][0][0]) ? tit = hint[key][0][0] : "";
            tit += "\n";
            for (var shft in hint[key]) {
                if (!hint[key][shft]) alert(key + ":" +shft);
                if (shft > 0 && hint[key][shft][0]) {
                    var skey = (glSkeys[shft]) ? " + " + glSkeys[shft] + " - " : "";
                    tit += "\n" + skey + hint[key][shft][0];
                }
            }
        } else if (hint[key]) {
            tit = hint[key];
        }
        return tit;
    },


    populateColorList: function(selectId, def) {
        if (selectId === undefined) return;
        if (def === undefined) def = "FFF";
        var colorKey = 'val_kbx_colors';
        var savedColors = vbRead(colorKey);
        if (!savedColors) {
            savedColors = "000:666:999:FFF:F00:F60:FF0:0F0:0FF:00F:90C:F0F";
            vbWrite(colorKey, savedColors);
        }
        var currentColor = vbRead(selectId);
        if (!currentColor) {
            currentColor = def;
            vbWrite(selectId, currentColor);
        }
        ModuleSystem.populateList(selectId, colorKey, selectId);
        var colorSelect = document.getElementById(selectId);
        if (colorSelect) {
            var options = colorSelect.options;
            for (var i = 0; i < options.length; i++) {
                options[i].style.background = '#' + options[i].value;
                options[i].innerHTML = '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
            }
        }
    },
    
    // Универсальная функция для работы со списками
    populateList: function(selectId, listValues) {
        // selectId - ID элемента select
        // listValues - массив объектов {key, val} или строка с разделителем ":"
        
        var selectElement = document.getElementById(selectId);
        if (!selectElement) return;
        
        var currentValue = vbRead(selectId);
        
        // Если передана строка - разбиваем на массив
        if (typeof listValues === 'string') {
            listValues = vbRead(listValues);
            var values = listValues.split(':');
            listValues = [];
            for (var i = 0; i < values.length; i++) {
                if (values[i] && values[i].length > 0) {
                    listValues.push({key: values[i], val: values[i]});
                }
            }
        }
        
        // Очищаем список
        while (selectElement.options.length > 0) {
            selectElement.remove(0);
        }
        
        // Добавляем новые options
        for (var i = 0; i < listValues.length; i++) {
            var key = listValues[i].key;
            var val = listValues[i].val || key;
            var hint = listValues[i].hint || "";
            
            if (key && key.length > 0) {
                var option = new Option(val, key);
                option.title = hint;
                option.selected = (currentValue == key);
                if (currentValue == key && hint) selectElement.title = hint;
                selectElement.options.add(option);
            }
        }
        //do_save(selectElement);
    },
    
    // Функция для добавления значения в список
    addToList: function(valueKey, currentKey, newValue) {
        var listValues = vbRead(valueKey);
        
        // Если список пустой, создаем новый
        if (!listValues || listValues === "") {
            listValues = newValue;
        } else {
            // Добавляем значение если его еще нет
            var values = listValues.split(":");
            if (!inArray(newValue, values)) {
                values.push(newValue);
                listValues = values.join(":");
            }
        }
        
        vbWrite(valueKey, listValues);
        vbWrite(currentKey, newValue);
    },
    
    // Функция для удаления значения из списка
    removeFromList: function(valueKey, currentKey, valueToRemove) {
        var listValues = vbRead(valueKey);
        if (!listValues) return;
        
        var values = listValues.split(":");
        var newValues = [];
        
        for (var i = 0; i < values.length; i++) {
            if (values[i] !== valueToRemove) {
                newValues.push(values[i]);
            }
        }
        
        vbWrite(valueKey, newValues.join(":"));
        vbWrite(currentKey, "");
    },
    
    // Функция для очистки списка
    clearList: function(valueKey, currentKey) {
        vbWrite(valueKey, "");
        vbWrite(currentKey, "");
    },

    // === Стилизация кнопок профилей состояний ===
    updateStateButtons: function() {
        for (var n = 1; n <= 3; n++) {
            var btn = document.getElementById('btn_state_' + n);
            var stval = document.getElementById('val_state_' + n);
            var val = vbRead('val_state_' + n);
            setVal(stval, val);
            if (!btn || !stval) continue;

            // Получаем сохранённый профиль
            var saved = val ? val : '';
            // Получаем текущие открытые вкладки
            var opened = [];
            for (var i = 0; i < this.availableModules.length; i++) {
                var moduleName = this.availableModules[i];
                var bodyElement = document.getElementById(moduleName);
                if (bodyElement && bodyElement.style.display !== "none") {
                    opened.push(moduleName);
                }
            }
            var openedStr = opened.sort().join(':');

            // Проверяем активный профиль
            //var state_last = document.getElementById('state_last');
            //var isActive = state_last && state_last.value == n;

            // Сравниваем открытые вкладки с профилем
            var isMatch = saved && (openedStr == saved);

            // Стилизация
            if (isMatch) {
                btn.style.background = '#444'; // тёмный фон
                btn.style.color = '#fff';
            } else {
                btn.style.background = '';
                btn.style.color = '';
            }
        }
    },

    // === Чередование классов для body модулей ===
    updateBodyOddEvenClasses: function() {
        var container = document.getElementById('divscroll');
        if (!container) return;
        
            // Чередуем по видимым body
        var allBodies = document.getElementsByTagName('div');
        var oddcnt = 0;
        for (var i = 0; i < allBodies.length; i++) {
            var el = allBodies[i];
            if (el.id.substring(0,4)=="cat_"){
                if (el.style.display == "none") continue;
                var cls = (oddcnt % 2 === 0) ? 'cat_body_even' : 'cat_body_odd';
                el.className = 'cat_body ' + cls;
                if (this.showTitles || this.moveMode) {
                    var title = document.getElementById('title_'+el.id);
                    title.className = 'cat_title cat_arrow_dn ' + cls;
                }
                oddcnt++;
            }
        }
    }
};

// Глобальные функции-обертки для совместимости
function cat_state(state, shft) {
    if (typeof ModuleSystem !== 'undefined' && ModuleSystem.manageState) {
        ModuleSystem.manageState(state, shft);
    }
}

function do_move(tit, dir) {
    if (typeof ModuleSystem === 'undefined') return;
    
    if (tit === "reset") {
        ModuleSystem.resetModulesOrder();
    } else {
        // Извлекаем имя модуля из ID заголовка
        var moduleName = tit.substring(0, 6) === 'title_' ? tit.substring(6) : tit;
        ModuleSystem.moveModule(moduleName, dir);
    }
}

function keyDN(el) {
    if (event.type == "keydown") {
        ModuleSystem.hideMenu(false, 99);
    }
}

// ===== УТИЛИТЫ ДЛЯ МОДУЛЕЙ =====

// 🔧 Безопасные функции локализации для модулей
// Эти функции автоматически проверяют доступность переменных lang/hint
// и возвращают fallback значения если переменные недоступны

function getLang(key, defaultValue) {
    if (typeof window.lang !== 'undefined' && window.lang[key]) {
        return window.lang[key];
    }
    if (typeof lang !== 'undefined' && lang[key]) {
        return lang[key];
    }
    return defaultValue !== undefined ? defaultValue : key;
}

function getHint(key, defaultValue) {
    if (typeof window.hint !== 'undefined' && window.hint[key]) {
        return window.hint[key];
    }
    if (typeof hint !== 'undefined' && hint[key]) {
        return hint[key];
    }
    return defaultValue !== undefined ? defaultValue : '';
}

function getBtns(key, defaultValue) {
    if (typeof window.btns !== 'undefined' && window.btns[key]) {
        return window.btns[key];
    }
    if (typeof btns !== 'undefined' && btns[key]) {
        return btns[key];
    }
    return defaultValue !== undefined ? defaultValue : '';
}


// Инициализация системы теперь вызывается явно из OnLoad() в htmlsys.js
// Автоматическая инициализация отключена чтобы избежать дублирования
function initModuleSystem() {
    if (typeof ShowCat !== 'undefined' && typeof includeHTML !== 'undefined' && document.getElementById('divscroll')) {
        ModuleSystem.init();
    } else {
        setTimeout(initModuleSystem, 10);
    }
}

window.ColorList = ModuleSystem.populateColorList;

// initModuleSystem(); // ← Отключена автоматическая инициализация 

/*
===== ДОКУМЕНТАЦИЯ ПО ПРОСТЫМ ФУНКЦИЯМ ДЛЯ МОДУЛЕЙ =====

📖 ФУНКЦИИ КЛАВИШ:
   var shiftState = Shift();     // Получить состояние клавиш (0-7)
   if (isShift()) { ... }        // Shift нажат?
   if (isCtrl()) { ... }         // Ctrl нажат?
   if (isAlt()) { ... }          // Alt нажат?

🌍 ФУНКЦИИ ЛОКАЛИЗАЦИИ (безопасные для модулей):
   var text = getLang('button_scan_colors', 'Поиск');     // Получить текст локализации
   var tooltip = getHint('buttons_select_colors', '');    // Получить подсказку
   var buttonText = getBtns('btn_check_length', '');      // Получить текст кнопки

💡 ПРИМЕР ИСПОЛЬЗОВАНИЯ В МОДУЛЕ:
   function myModuleAction() {
       var shiftState = Shift();
       if (isAlt(shiftState)) {
           // Выполнить расширенное действие
           var confirmText = getLang('confirm_extended_action', 'Выполнить расширенное действие?');
           if (confirm(confirmText)) {
               // действие
           }
       } else {
           // Обычное действие
           var buttonTitle = getHint('my_button', 'Нажмите для выполнения');
           document.getElementById('myButton').title = buttonTitle;
       }
   }

⚠️  ВАЖНО: В модулях всегда используйте getLang(), getHint(), getBtns() 
    вместо прямых обращений к lang[], hint[], btns[]
*/ 

function ShowInnerBlock(blockId) {
    var body = document.getElementById(blockId);
    var title = document.getElementById('tit_' + blockId);
    if (!body || !title) return;
    var isOpen = body.style.display !== 'none';
    var newState = !isOpen;
    body.style.display = newState ? 'block' : 'none';
    title.className = newState ? 'cat_tit cat_arrow_up' : 'cat_tit cat_arrow_dn';
    vbWrite(blockId, newState ? 'true' : 'false');
} 